@@ -8,6 +8,7 @@ use std::sync::{Arc, LazyLock, RwLock};
 use itertools::Either;
 use rustc_hash::{FxHashMap, FxHashSet};
 use thiserror::Error;
+use tracing::trace;
 use url::{ParseError, Url};
 
 use uv_pep508::{Scheme, VerbatimUrl, VerbatimUrlError, split_scheme};
@@ -81,22 +82,21 @@ impl schemars::JsonSchema for IndexUrl {
 }
 
 impl IndexUrl {
+    #[inline]
+    fn inner(&self) -> &VerbatimUrl {
+        match self {
+            Self::Pypi(url) | Self::Url(url) | Self::Path(url) => url,
+        }
+    }
+
     /// Return the raw URL for the index.
     pub fn url(&self) -> &DisplaySafeUrl {
-        match self {
-            Self::Pypi(url) => url.raw(),
-            Self::Url(url) => url.raw(),
-            Self::Path(url) => url.raw(),
-        }
+        self.inner().raw()
     }
 
     /// Convert the index URL into a [`DisplaySafeUrl`].
     pub fn into_url(self) -> DisplaySafeUrl {
-        match self {
-            Self::Pypi(url) => url.to_url(),
-            Self::Url(url) => url.to_url(),
-            Self::Path(url) => url.to_url(),
-        }
+        self.inner().to_url()
     }
 
     /// Return the redacted URL for the index, omitting any sensitive credentials.
@@ -139,21 +139,13 @@ impl IndexUrl {
 
 impl Display for IndexUrl {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Pypi(url) => Display::fmt(url, f),
-            Self::Url(url) => Display::fmt(url, f),
-            Self::Path(url) => Display::fmt(url, f),
-        }
+        Display::fmt(self.inner(), f)
     }
 }
 
 impl Verbatim for IndexUrl {
     fn verbatim(&self) -> Cow<'_, str> {
-        match self {
-            Self::Pypi(url) => url.verbatim(),
-            Self::Url(url) => url.verbatim(),
-            Self::Path(url) => url.verbatim(),
-        }
+        self.inner().verbatim()
     }
 }
 
@@ -203,11 +195,7 @@ impl serde::ser::Serialize for IndexUrl {
     where
         S: serde::ser::Serializer,
     {
-        match self {
-            Self::Pypi(url) => url.without_credentials().serialize(serializer),
-            Self::Url(url) => url.without_credentials().serialize(serializer),
-            Self::Path(url) => url.without_credentials().serialize(serializer),
-        }
+        self.inner().without_credentials().serialize(serializer)
     }
 }
 
@@ -248,11 +236,7 @@ impl From<VerbatimUrl> for IndexUrl {
 
 impl From<IndexUrl> for DisplaySafeUrl {
     fn from(index: IndexUrl) -> Self {
-        match index {
-            IndexUrl::Pypi(url) => url.to_url(),
-            IndexUrl::Url(url) => url.to_url(),
-            IndexUrl::Path(url) => url.to_url(),
-        }
+        index.inner().to_url()
     }
 }
 
@@ -260,11 +244,7 @@ impl Deref for IndexUrl {
     type Target = Url;
 
     fn deref(&self) -> &Self::Target {
-        match self {
-            Self::Pypi(url) => url,
-            Self::Url(url) => url,
-            Self::Path(url) => url,
-        }
+        self.inner()
     }
 }
 
@@ -457,6 +437,14 @@ impl<'a> IndexLocations {
     pub fn cache_index_credentials(&self) {
         for index in self.known_indexes() {
             if let Some(credentials) = index.credentials() {
+                trace!(
+                    "Read credentials for index {}",
+                    index
+                        .name
+                        .as_ref()
+                        .map(ToString::to_string)
+                        .unwrap_or_else(|| index.url.to_string())
+                );
                 let credentials = Arc::new(credentials);
                 uv_auth::store_credentials(index.raw_url(), credentials.clone());
                 if let Some(root_url) = index.root_url() {
@@ -470,7 +458,7 @@ impl<'a> IndexLocations {
     pub fn simple_api_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
         for index in &self.indexes {
             if index.url() == url {
-                return index.cache_control.as_ref()?.api.as_deref();
+                return index.simple_api_cache_control();
             }
         }
         None
@@ -480,7 +468,7 @@ impl<'a> IndexLocations {
     pub fn artifact_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
         for index in &self.indexes {
             if index.url() == url {
-                return index.cache_control.as_ref()?.files.as_deref();
+                return index.artifact_cache_control();
             }
         }
         None
@@ -623,7 +611,7 @@ impl<'a> IndexUrls {
     pub fn simple_api_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
         for index in &self.indexes {
             if index.url() == url {
-                return index.cache_control.as_ref()?.api.as_deref();
+                return index.simple_api_cache_control();
             }
         }
         None
@@ -633,7 +621,7 @@ impl<'a> IndexUrls {
     pub fn artifact_cache_control_for(&self, url: &IndexUrl) -> Option<&str> {
         for index in &self.indexes {
             if index.url() == url {
-                return index.cache_control.as_ref()?.files.as_deref();
+                return index.artifact_cache_control();
             }
         }
         None
@@ -723,6 +711,8 @@ impl IndexCapabilities {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::{IndexCacheControl, IndexFormat, IndexName};
+    use uv_small_str::SmallString;
 
     #[test]
     fn test_index_url_parse_valid_paths() {
@@ -816,4 +806,88 @@ mod tests {
         assert_eq!(index_urls.simple_api_cache_control_for(&url3), None);
         assert_eq!(index_urls.artifact_cache_control_for(&url3), None);
     }
+
+    #[test]
+    fn test_pytorch_default_cache_control() {
+        // Test that PyTorch indexes get default cache control from the getter methods
+        let indexes = vec![Index {
+            name: Some(IndexName::from_str("pytorch").unwrap()),
+            url: IndexUrl::from_str("https://download.pytorch.org/whl/cu118").unwrap(),
+            cache_control: None, // No explicit cache control
+            explicit: false,
+            default: false,
+            origin: None,
+            format: IndexFormat::Simple,
+            publish_url: None,
+            authenticate: uv_auth::AuthPolicy::default(),
+            ignore_error_codes: None,
+        }];
+
+        let index_urls = IndexUrls::from_indexes(indexes.clone());
+        let index_locations = IndexLocations::new(indexes, Vec::new(), false);
+
+        let pytorch_url = IndexUrl::from_str("https://download.pytorch.org/whl/cu118").unwrap();
+
+        // IndexUrls should return the default for PyTorch
+        assert_eq!(index_urls.simple_api_cache_control_for(&pytorch_url), None);
+        assert_eq!(
+            index_urls.artifact_cache_control_for(&pytorch_url),
+            Some("max-age=365000000, immutable, public")
+        );
+
+        // IndexLocations should also return the default for PyTorch
+        assert_eq!(
+            index_locations.simple_api_cache_control_for(&pytorch_url),
+            None
+        );
+        assert_eq!(
+            index_locations.artifact_cache_control_for(&pytorch_url),
+            Some("max-age=365000000, immutable, public")
+        );
+    }
+
+    #[test]
+    fn test_pytorch_user_override_cache_control() {
+        // Test that user-specified cache control overrides PyTorch defaults
+        let indexes = vec![Index {
+            name: Some(IndexName::from_str("pytorch").unwrap()),
+            url: IndexUrl::from_str("https://download.pytorch.org/whl/cu118").unwrap(),
+            cache_control: Some(IndexCacheControl {
+                api: Some(SmallString::from("no-cache")),
+                files: Some(SmallString::from("max-age=3600")),
+            }),
+            explicit: false,
+            default: false,
+            origin: None,
+            format: IndexFormat::Simple,
+            publish_url: None,
+            authenticate: uv_auth::AuthPolicy::default(),
+            ignore_error_codes: None,
+        }];
+
+        let index_urls = IndexUrls::from_indexes(indexes.clone());
+        let index_locations = IndexLocations::new(indexes, Vec::new(), false);
+
+        let pytorch_url = IndexUrl::from_str("https://download.pytorch.org/whl/cu118").unwrap();
+
+        // User settings should override defaults
+        assert_eq!(
+            index_urls.simple_api_cache_control_for(&pytorch_url),
+            Some("no-cache")
+        );
+        assert_eq!(
+            index_urls.artifact_cache_control_for(&pytorch_url),
+            Some("max-age=3600")
+        );
+
+        // Same for IndexLocations
+        assert_eq!(
+            index_locations.simple_api_cache_control_for(&pytorch_url),
+            Some("no-cache")
+        );
+        assert_eq!(
+            index_locations.artifact_cache_control_for(&pytorch_url),
+            Some("max-age=3600")
+        );
+    }
 }
